package controllers

import (
	"fmt"
	"net/http"

	"hello-api/internal/utils"

	"gitlab.bit9.local/psc/ems/hinting/pkg/hints"

	"hello-api/internal/app/middleware"
	"hello-api/internal/app/parsers/deviceresponseheader"

	"github.com/gin-gonic/gin"
)

const (
	HelloResponseCode = 0x201d
)

type HelloController struct {
	responseBodyEncoder deviceresponseheader.ResponseHeaderEncoder
}

// NewHelloController -- builder method for hello controller
func NewHelloController() HelloController {
	return HelloController{responseBodyEncoder: deviceresponseheader.NewResponseHeaderEncoder()}
}

func (controller *HelloController) WithEncoder(responseBodyEncoder deviceresponseheader.ResponseHeaderEncoder) *HelloController {
	controller.responseBodyEncoder = responseBodyEncoder
	return controller
}

func (controller *HelloController) DoHello(c *gin.Context) {
	hintResult, err := utils.ContextGet[*hints.HintResult](c, middleware.HintsName)
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to get hints from context: %w", err))
		return
	}
	version, err := utils.ContextGet[int](c, middleware.VersionName)
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to get version from context: %w", err))
		return
	}

	hintsArray := make([]int, 0)
	if hintResult.Hints != nil {
		hintsArray = hintResult.Hints.ToIntArray()
	}
	header := deviceresponseheader.NewDeviceResponseHeader(int64(version), HelloResponseCode, true,
		0, "Success.", hintsArray, hintResult.LastUpdateTime, hintResult.LastLiveQueryTime)
	body, err := controller.responseBodyEncoder.Encode(header)
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to encode response body: %w", err))
		return
	}
	c.Set("response", header)
	_, err = c.Writer.Write([]byte(body))
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to write response body: %w", err))
	}
}

type postRouterGroup interface {
	POST(string, ...gin.HandlerFunc) gin.IRoutes
}

func (controller *HelloController) AddToRouter(routerGroup postRouterGroup, lctPublisherMiddleware, hintingMiddleware gin.HandlerFunc) {
	routerGroup.POST("/hello", middleware.DeviceContextMiddleware, middleware.DeviceRequestHeaderMiddleware, lctPublisherMiddleware, hintingMiddleware, controller.DoHello)
}
