package middleware

import (
	"fmt"
	"net/http"
	"time"

	"hello-api/internal/logger"

	"hello-api/internal/utils"

	"hello-api/internal/lct"

	"carbonblack.com/protocols/psc/devicelastcontacttime"
	"carbonblack.com/protocols/psc/ems/server/deviceauth"
	"github.com/gin-gonic/gin"
	"github.com/opentracing/opentracing-go"
)

func LctPublisherMiddleware(publisher lct.LctProducer) gin.HandlerFunc {
	handler := func(context *gin.Context) {
		func() {
			sp := opentracing.SpanFromContext(context.Request.Context())

			if sp != nil {
				childSp := opentracing.StartSpan("lct-publish", opentracing.ChildOf(sp.Context()))
				defer childSp.Finish()
			}

			deviceContext, err := utils.ContextGet[*deviceauth.Context](context, DeviceContextHeader)
			if err != nil {
				_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to get device context header: %w", err))
				return
			}
			deviceLastContactTime := &devicelastcontacttime.DeviceLastContactTime{
				OrgKey:   deviceContext.Device.OrgKey,
				DeviceId: deviceContext.Device.GetId(), LastContactTimestamp: time.Now().UnixMilli(), DeregisterAfterTime: 0,
			}
			logger.BaseLogger().Debug(fmt.Sprintf("Device last contact time %+v", deviceLastContactTime))

			if err = publisher.Publish(deviceLastContactTime); err != nil {
				_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to publish to last contact updater: %w", err))
				return
			}
		}()

		context.Next()
	}
	return handler
}
