package devicecontext

import (
	"encoding/base64"
	"fmt"
	"testing"

	deviceauthpb "carbonblack.com/protocols/psc/ems/server/deviceauth"
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"google.golang.org/protobuf/proto"
)

func TestDeviceAuthResponseParsing(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "protobuf device auth response header")
}

var _ = Describe("device context parser", func() {
	parser := NewDeviceAuthResponseParser()
	Context("with valid inputs", func() {
		It("accepts valid input", func() {
			device := &deviceauthpb.Device{
				Id:             1,
				DeploymentType: deviceauthpb.DeploymentType_ENDPOINT,
				DeviceType:     deviceauthpb.DeviceType_LINUX,
				OrgKey:         "ORGKEY",
			}
			group := &deviceauthpb.DeviceGroup{
				Id:    1,
				OrgId: 2,
			}
			org := &deviceauthpb.Organization{
				Id:     1,
				OrgKey: "ORGKEY",
			}
			input := &deviceauthpb.Context{Device: device, DeviceGroup: group, Organization: org}
			marshalled, err := proto.Marshal(input)
			Expect(err).To(BeNil())
			b64Encoded := base64.StdEncoding.EncodeToString(marshalled)
			resp, err2 := parser.ParseResponse(b64Encoded)
			Expect(err2).To(BeNil())
			Expect(resp).ToNot(BeNil()) //
		})
	})

	Context("with invalid inputs", func() {
		It("returns error when given empty input", func() {
			context, err := parser.ParseResponse("")
			Expect(err).ToNot(BeNil())
			Expect(context).To(BeNil())
		})

		It("returns error when given erroneous input", func() {
			context, err := parser.ParseResponse("DEADBEEF")
			Expect(err).ToNot(BeNil())
			Expect(context).To(BeNil())
		})

		It("returns error with missing device info", func() {
			group := &deviceauthpb.DeviceGroup{
				Id:    1,
				OrgId: 2,
			}
			org := &deviceauthpb.Organization{
				Id:     1,
				OrgKey: "ORGKEY",
			}
			input := &deviceauthpb.Context{DeviceGroup: group, Organization: org}
			marshalled, err := proto.Marshal(input)
			Expect(err).To(BeNil())
			b64Encoded := base64.StdEncoding.EncodeToString(marshalled)
			_, err = parser.ParseResponse(b64Encoded)
			Expect(err).ToNot(BeNil())
			Expect(fmt.Sprintln(err)).To(ContainSubstring("no device included in device context header"))
		})
	})
})
