// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package config

import "sync"

// singleton configuration instance
var (
	onceLogger           sync.Once
	loggerConfigInstance *LoggerConfig
)

type LoggerConfig struct {
	LogLevel  string
	CommitSHA string
}

func GetLoggerConfigInstance() *LoggerConfig {
	onceLogger.Do(func() {
		loggerConfigInstance = newLoggerDefaultConfig(true)
	})
	return loggerConfigInstance
}

func newLoggerDefaultConfig(loadFromEnvironment bool) *LoggerConfig {
	cfg := LoggerConfig{
		LogLevel:  NewConfigParamWithDefault(envVarKeyLogLevel, defaultLogLevel).Get(loadFromEnvironment),
		CommitSHA: NewConfigParamWithDefault(envVarKeyCommitSHA, "").Get(loadFromEnvironment),
	}

	return &cfg
}
