// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package utils

import (
	"fmt"
	"os"
	"strings"
	"time"

	"hello-api/internal/logger"

	"github.com/gin-gonic/gin"
)

// CompareTimePointers -- Compare the values of two pointers to timestamps, return true if same time or both nil
func CompareTimePointers(one, two *time.Time) (equal bool) {
	if one != nil && two != nil {
		return one.Equal(*two)
	}
	if one == nil && two == nil {
		return true
	}
	return false
}

// FlagEmpty -- Trim and return supplied string, use value of "*UNSET*" if empty string
func FlagEmpty(value string) string {
	check := strings.TrimSpace(value)
	if check == "" {
		check = "*UNSET*"
	}
	return check
}

func ContextGet[T any](ctx *gin.Context, key string) (t T, err error) {
	if temp, found := ctx.Get(key); found {
		var ok bool
		if t, ok = temp.(T); ok {
			return t, nil
		} else {
			return t, fmt.Errorf("failed to convert context value for key %s to %v", key, t)
		}
	}
	return t, fmt.Errorf("failed to find key %s in context", key)
}

// ContextGetUnixMilliFromPtr -- Safely get a timestamp in millis from a context timestamp, or nil if the pointer is nil
func ContextGetUnixMilliFromPtr(ctx *gin.Context, key string) *int64 {
	t, err := ContextGet[*time.Time](ctx, key)
	if err != nil {
		logger.BaseLogger().Debug(fmt.Sprintf("error extracting %s from context", key))
		return nil
	} else if t == nil {
		return nil
	}

	timestamp := t.UnixMilli()
	return &timestamp
}

// EnvOrDefault -- Attempt to get environment variable, or use fallback if not defined
func EnvOrDefault(key, fallback string) string {
	value, found := os.LookupEnv(key)
	if !found {
		value = fallback
	}
	return value
}

func SafeTimeString(value *time.Time) string {
	if value == nil {
		return "nil"
	} else {
		return value.String()
	}
}
